import { Control } from "./Control";
import { RemoteFuncs } from "./Remote";

export type ObjectType = "Strip" | "Bus";

export abstract class VMObject {
  constructor(
    public control: Control,
    public objectType: ObjectType,
    public objectId?: number
  ) {
    this.initialize();
  }

  abstract initialize(): void;

  protected buildParameterString(...path: (string | number)[]) {
    let string = `${this.objectType}${
      this.objectId !== undefined ? `[${this.objectId}]` : ""
    }`;
    for (let pathId = 0; pathId < path.length; pathId++) {
      const parameter = path[pathId];
      let idForParameter: number | undefined = undefined;
      if (typeof path[pathId + 1] === "number") {
        idForParameter = path[pathId + 1] as number;
        pathId++;
      }
      string += `.${parameter}${
        idForParameter !== undefined ? `[${idForParameter}]` : ""
      }`;
    }
    return string;
  }

  protected getParamValueString(parameter: string): [number, string] {
    const buffer = Buffer.alloc(512);
    const result = this.control.run(
      "VBVMR_GetParameterStringA",
      parameter,
      buffer
    );
    return [result, buffer.toString()];
  }
  protected getParamValueBoolean(parameter: string): [number, boolean] {
    const buffer = Buffer.alloc(512);
    const result = this.control.run(
      "VBVMR_GetParameterStringA",
      parameter,
      buffer
    );
    return [result, buffer.subarray(0, 5).toString().startsWith("1")];
  }
  protected getParamValueNumber(parameter: string): [number, number] {
    const buffer = Buffer.alloc(512);
    const result = this.control.run(
      "VBVMR_GetParameterFloat",
      parameter,
      buffer
    );
    return [result, buffer.readFloatLE()];
  }

  protected setParamValueString(parameter: string, value: string): number {
    const result = this.control.run(
      "VBVMR_SetParameterStringA",
      parameter,
      value
    );
    return result;
  }
  protected setParamValueBoolean(parameter: string, value: boolean): number {
    const result = this.control.run(
      "VBVMR_SetParameterFloat",
      parameter,
      value ? 1 : 0
    );
    return result;
  }
  protected setParamValueNumber(parameter: string, value: number): number {
    const result = this.control.run(
      "VBVMR_SetParameterFloat",
      parameter,
      value
    );
    return result;
  }
  protected setParameters(script: string): number {
    const result = this.control.run("VBVMR_SetParameters", script);
    return result;
  }
}
